#import <Foundation/Foundation.h>
#import "MVKeyChain.h"


static MVKeyChain *MVDefaultKeyChain = nil;

@implementation MVKeyChain

+ (MVKeyChain *) defaultKeyChain {

    if (MVDefaultKeyChain == nil) {
        MVDefaultKeyChain = [[self alloc] init];
    }
    
    return MVDefaultKeyChain;
}

- (void) dealloc {

    if (MVDefaultKeyChain == self) {
        MVDefaultKeyChain = nil;
    }
    [super dealloc];

    return;
}

- (void) setInternetPassword:(NSString *) password forServer:(NSString *) server account:(NSString *) account port:(unsigned short) port protocol:(SecProtocolType) protocol {
    OSStatus err = 0;
    UInt32 oldPassLength = 0;
    void *oldPassword = NULL;
    SecKeychainItemRef passRef;

    NSAssert([server length], @"server name must not be empty");
    NSAssert([account length], @"account name must not be empty");

    /* Check wether there is alrready one. */
    err = SecKeychainFindInternetPassword(
                NULL,
                strlen([server UTF8String]), [server UTF8String],
                0, NULL, // securityDomain
                strlen([account UTF8String]), [account UTF8String],
                0, NULL, // path
                port,
                protocol,
                'dflt', // kSecAuthTypeDefault
                &oldPassLength, &oldPassword,
                &passRef);

    if ([password length]) {
        if (err) {
            /* Set a new one. */
            SecKeychainAddInternetPassword(
                NULL,
                strlen([server UTF8String]), [server UTF8String],
                0, NULL, // securityDomain
                strlen([account UTF8String]), [account UTF8String],
                0, NULL, // path
                port,
                protocol,
                'dflt', // kSecAuthTypeDefault
                strlen([password UTF8String]), [password UTF8String],
                NULL);
        } else if (strncmp([password UTF8String], oldPassword, oldPassLength)) {
            /* Change the existing one if different. */
            SecKeychainItemModifyContent(passRef, NULL, strlen([password UTF8String]), [password UTF8String]);
        }
    } else {
        if (! err) {
            /* Remove a existing one. */
            SecKeychainItemDelete(passRef);
            CFRelease(passRef);
        }
    }        
    
    return;
}

- (NSString *) internetPasswordForServer:(NSString *) server account:(NSString *) account port:(unsigned short) port protocol:(SecProtocolType) protocol {
    OSStatus err = 0;
    UInt32 passLength = 0;
    void *password = NULL;

    NSAssert([server length], @"server name must not be empty");
    NSAssert([account length], @"account name must not be empty");

    err = SecKeychainFindInternetPassword(
                NULL,
                strlen([server UTF8String]), [server UTF8String],
                0, NULL, // securityDomain
                strlen([account UTF8String]), [account UTF8String],
                0, NULL, // path
                port,
                protocol,
                'dflt', // kSecAuthTypeDefault
                &passLength, &password,
                NULL); // KeychainItemReference
    if (err) {
        return nil;
    }

    return [NSString stringWithCString:password length:passLength];
}

@end